// =======================================================================
// 
//       Filename:  returnstatement.cpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  18.06.2010 12:37:56
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================

#include "returnstatement.hpp"
#include "../elementstack.hpp"
#include "../exceptions/returnexception.hpp"
#include "../types/void.hpp"
#include "objectwrapper.hpp"

using namespace boost;

namespace wesl {
    namespace elements {
        ReturnStatement::ReturnStatement(ElementStack& parent)
            : Element(parent),
            Callable(this)
        {

        }

        ReturnStatement::ReturnStatement(ElementStack& parent,
                                         ElementPtr element)
            : Element(parent), 
            Callable(this),
            m_element(element)
        {

        }

        void ReturnStatement::call() {
            if(m_element->is(Element::RESOLVABLE)) {
                ResolvablePtr ptr = 
                    dynamic_pointer_cast<Resolvable>(m_element);

                throw exceptions::ReturnException(ptr->resolve(),
                    "return statement outside of function",
                    parent().sourcePath(),
                    line());
            }

            throw exceptions::ProcessingError(
                    "return: Element is not resolvable",
                    parent().sourcePath(),
                    line());
        }

        void ReturnStatement::setElement(ElementPtr element)
        {
            m_element = element;
        }
    }
}
