// =======================================================================
// 
//       Filename:  scriptelement.cpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  20.04.2010 00:56:49
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================

#include "script.hpp"
#include "../elementstack.hpp"
#include "../exceptions/processingerror.hpp"
#include "../types/string.hpp"
#include "../process/process.hpp"
#include <sstream>
#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

using namespace std;
using namespace boost;

namespace wesl {
    namespace elements {
        shared_ptr<ElementStack::VariableMap>
            buildCompleteVariableMap(Element& object)
        {

            list<ElementStack*> path;

            ElementStack* current = &object.parent();

            while(1) {
                path.push_front(current);

                if(current->hasParent())
                    current = &current->parent();
                else
                    break;
            }

            shared_ptr<ElementStack::VariableMap> cMapPtr(
                    new ElementStack::VariableMap());

            ElementStack::VariableMap::iterator staticVarMapIter;
            for(staticVarMapIter = 
                    ElementStack::STATIC_VARIABLE_MAP.begin();
                    staticVarMapIter != 
                    ElementStack::STATIC_VARIABLE_MAP.end();
                    ++staticVarMapIter)
            {
                (*cMapPtr)[staticVarMapIter->first] =
                    staticVarMapIter->second;
            }

            foreach(ElementStack* es, path) {
                ElementStack::VariableMap& m =
                    es->dynamicVariableMap();

                ElementStack::VariableMap::iterator iter;

                for(iter = m.begin(); iter != m.end(); ++iter)
                {
                    (*cMapPtr)[iter->first] = iter->second;
                }
            }

            return cMapPtr;
        }

        Script::Script(ElementStack& parent)
            : Text(parent),
            Resolvable(this),
            Callable(this)
        {

        }

        Script::Script(ElementStack& parent,
                       const string& spec)
            : Text(parent), Resolvable(this),
            Callable(this),
            mSpec(spec)
        {

        }

        Script::~Script() {
        }

        string Script::specification() const
        {
            return mSpec;
        }

        void Script::setSpecification(const string& spec)
        {
            mSpec = spec;
        }

        void Script::print(ostream& stream) {
            if(RULES[specification()].empty()) {
                throw exceptions::ProcessingError(
                        "no rule defined for: " +
                            specification(),
                            parent().sourcePath(),
                            line());
            }
            else {
                shared_ptr<ElementStack::VariableMap> cMap =
                    buildCompleteVariableMap(*this);
                ElementStack::VariableMap::iterator pageIter;

                for(pageIter = cMap->begin(); 
                        pageIter != cMap->end();
                        ++pageIter)
                {
                    string envVar("WP_");
                    envVar.append(pageIter->first);

                    setenv(envVar.c_str(), 
                            pageIter->second->toString().c_str(), 1);
                }

                string scriptSpec = RULES[
                    specification()];

                process::Process process(scriptSpec);
                pid_t ret = process.launch();

                if (ret >= 0)
                {
                    char ch;

                    while(*this >> ch) {
                        process << ch;
                    }

                    process.closeWriteChannel();

                    while(process >> ch) {
                        stream << ch;
                    }

                    process.closeReadChannel();
                }

                for(pageIter = cMap->begin(); 
                        pageIter != cMap->end();
                        ++pageIter)
                {
                    string envVar("WP_");
                    envVar.append(pageIter->first);

                    unsetenv(envVar.c_str());
                }
            }
        }

        types::ObjectPtr Script::resolve() {
            ostringstream os;

            print(os);

            types::StringPtr stringPtr(
                    new types::String(os.str()));

            return stringPtr;
        }

        void Script::call() {
            if(RULES[specification()].empty()) {
                throw exceptions::ProcessingError(
                        "no rule defined for: " +
                            specification(),
                            parent().sourcePath(),
                            line());
            }
            else {
                shared_ptr<ElementStack::VariableMap> cMap =
                    buildCompleteVariableMap(*this);
                ElementStack::VariableMap::iterator pageIter;

                for(pageIter = cMap->begin(); 
                        pageIter != cMap->end();
                        ++pageIter)
                {
                    string envVar("WP_");
                    envVar.append(pageIter->first);

                    setenv(envVar.c_str(), 
                            pageIter->second->toString().c_str(), 1);
                }

                string scriptSpec = RULES[
                    specification()];

                process::Process process(scriptSpec);
                pid_t ret = process.launch();

                if (ret >= 0)
                {
                    char ch;

                    while(*this >> ch) {
                        process << ch;
                    }

                    process.closeWriteChannel();

                    while(process >> ch) {
                        // do nothing
                    }

                    process.closeReadChannel();
                }

                for(pageIter = cMap->begin(); 
                        pageIter != cMap->end();
                        ++pageIter)
                {
                    string envVar("WP_");
                    envVar.append(pageIter->first);

                    unsetenv(envVar.c_str());
                }
            }
        }

        map<string,string> Script::RULES =
                                map<string,string>();
    }
}
