// =======================================================================
// 
//       Filename:  textelement.cpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  20.04.2010 00:53:56
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================

#include "text.hpp"
#include "../types/string.hpp"
#include "../utils/citerator.hpp"
#include "../exceptions/processingerror.hpp"
#include "../elementstack.hpp"
#include <stdexcept>

using namespace std;
using wesl::exceptions::ProcessingError;
namespace fs = boost::filesystem;

namespace wesl {
    namespace elements {
        Text::Text(ElementStack& parent)
            : Element(parent),
            Printable(this),
            mBegin(-1), mEnd(-1),
            mIterator(0), m_firstRead(true)
        {

        }

        Text::Text(ElementStack& parent, 
                    int begin, int end)
            : Element(parent),
            Printable(this),
            mBegin(begin), mEnd(end),
            mIterator(0), m_firstRead(true)
        {

        }

        Text::~Text() {
        }

        int Text::begin() const
        {
            return mBegin;
        }

        void Text::setBegin(int begin)
        {
            mBegin = begin;
        }

        int Text::end() const
        {
            return mEnd;
        }

        void Text::setEnd(int end)
        {
            mEnd = end;
        }

        void Text::setExternalFile(const fs::path& filePath)
        {
            m_externalFilePath = filePath;
        }

        fs::path Text::externalFile() const
        {
            return m_externalFilePath;
        }

        bool Text::hasExternalFile() const
        {
            return (!m_externalFilePath.empty());
        }

        void Text::print(std::ostream& stream) {
            char ch;

            while(*this >> ch) {
                stream << ch;
            }
        }

        bool Text::operator>>(char& ch) {
            return getChar(ch);
        }

        bool Text::getChar(char& ch)
        {
            if(!this->hasExternalFile() && begin() < 0)
                return false;
            else if(hasExternalFile() && m_firstRead) {
                m_input.open(externalFile().string().c_str());
                if(!m_input.good())
                    throw ProcessingError("unable to open: " + 
                            externalFile().string(),
                            parent().sourcePath(),
                            line());

                mIterator = new utils::CIterator(m_input);
            }
            else if(!hasExternalFile() && m_firstRead) {
                mIterator = &parent().iterator();
            }

            if(mIterator == 0)
                throw logic_error("No input iterator set");

            if(m_firstRead)
                while(mIterator->position() < begin())
                    mIterator->moveForward();

            if(mIterator->good()) {
                if(mIterator->position() < end() || end() == -1)
                {
                    ch = mIterator->currentChar();
                    mIterator->moveForward();

                    if(m_firstRead)
                        m_firstRead = false;

                    return true;
                }
                else if(m_firstRead) {
                    searchBegin();
                    return getChar(ch);
                }
            }
            else if(m_firstRead) {
                searchBegin();
                return getChar(ch);
            }

            if(hasExternalFile()) {
                m_input.close();
                delete mIterator;
            }

            mIterator = 0;
            m_firstRead = true;
            return false;
        }

        void Text::searchBegin() {
            mIterator->rewind();
            while(mIterator->position() < begin())
                    mIterator->moveForward();
        }
    }
}

bool operator<<(char& ch, wesl::elements::Text& element)
{
    return element >> ch;
}

