// =======================================================================
// 
//       Filename:  textelement.hpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  20.04.2010 00:40:31
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================


#ifndef  TEXTELEMENT_INC
#define  TEXTELEMENT_INC
#include "element.hpp"
#include "printable.hpp"
#include <boost/filesystem/path.hpp>

namespace wesl {
    // Forward declaration
    namespace utils {
        class CIterator;
    }

    namespace types {
        class Object;
        typedef boost::shared_ptr<Object> ObjectPtr;
    }

    namespace elements {
        class Text : public Element,
                     public Printable
        {
            public:
                Text(ElementStack& parent);

                Text(ElementStack& parent, 
                        int begin, int end);

                ~Text();

                int begin() const;

                void setBegin(int begin);

                int end() const;

                void setEnd(int end);

                void setExternalFile(const boost::filesystem::path& 
                                                            filePath);

                boost::filesystem::path externalFile() const;

                bool hasExternalFile() const;

                void print(std::ostream& stream);

                virtual bool operator>>(char& ch);

                friend Text& operator>>(char ch, Text& element);

            protected:
                virtual bool getChar(char& ch);

            private:
                void searchBegin();

            private:
                int mBegin;
                int mEnd;
                utils::CIterator* mIterator;
                std::ifstream m_input;
                boost::filesystem::path m_externalFilePath;
                bool m_firstRead;
        };

        typedef boost::shared_ptr<Text> TextPtr;
    }
}
#endif   // ----- #ifndef TEXTELEMENT_INC  -----
