// =======================================================================
// 
//       Filename:  varelement.cpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  20.04.2010 00:55:00
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================

#include "variablerecall.hpp"
#include "../elementstack.hpp"
#include "../exceptions/processingerror.hpp"
#include "../types/object.hpp"
#include <stdexcept>
#include <boost/shared_ptr.hpp>

using namespace std;
using namespace boost;

namespace wesl {
    namespace elements {
        VariableRecall::VariableRecall(ElementStack& parent)
            : Element(parent),
            Resolvable(this)
        {

        }

        VariableRecall::VariableRecall(ElementStack& parent, const string& name)
            : Element(parent),
              Resolvable(this),
              mName(name)
        {

        }

        string VariableRecall::name() const
           
        {
            return mName;
        }

        void VariableRecall::setName(const string& name)
           
        {
            mName = name;
        }

        bool VariableRecall::getChar(char& ch)
        {
            throw logic_error("Cannot read from a VariableRecall");
        }

        types::ObjectPtr VariableRecall::resolve() {
            shared_ptr<types::Object> object;

            ElementStack::VariableMap::iterator iter;
            ElementStack::VariableMap::iterator endIter;

            iter = 
                parent().lookupVariable(name());
            endIter = 
                ElementStack::STATIC_VARIABLE_MAP.end();

            if(iter == endIter)
                throw exceptions::ProcessingError(
                        "undefined variable: " +
                        name(),
                        this->parent().sourcePath(),
                        line()); 
            else
                object = iter->second;

            return object;
        }

        void VariableRecall::print(ostream& stream) {
            resolve()->print(stream);
        }
    }
}
