// =======================================================================
// 
//       Filename:  elementstack.hpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  24.05.2010 21:16:33
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================


#ifndef  ELEMENTSTACK_INC
#define  ELEMENTSTACK_INC
#include "elements/element.hpp"
#include "elements/printable.hpp"
#include "elements/resolvable.hpp"
#include "utils/citerator.hpp"
#include <boost/filesystem/path.hpp>
#include <boost/shared_ptr.hpp>
#include <list>
#include <fstream>
#include <map>

namespace wesl {
    namespace symbols {
        class Function;
        class Macro;
    }

    namespace types {
        class Object;
    }

    class ElementStack : public std::list<elements::ElementPtr>,
                         public elements::Element,
                         public elements::Printable,
                         public elements::Resolvable
    {
    public:
        typedef std::map<std::string,boost::shared_ptr<symbols::Function> > FunctionMap;
        typedef std::map<std::string,boost::shared_ptr<symbols::Macro> > MacroMap;
        typedef std::map<std::string,boost::shared_ptr<types::Object> > VariableMap;

        ElementStack();

        ElementStack(const boost::filesystem::path& sourcePath);

        void open();

        void close();

        utils::CIterator& iterator();

        boost::filesystem::path sourcePath() const;

        void setProcessed(bool state = true);

        bool processed() const;

        ElementStack& head();

        VariableMap::iterator lookupVariable(const std::string& name);

        VariableMap& dynamicVariableMap();
        const VariableMap& dynamicVariableMap() const;

        FunctionMap::iterator lookupFunction(const std::string& name);

        FunctionMap& dynamicFunctionMap();
        const FunctionMap& dynamicFunctionMap() const;

        MacroMap::iterator lookupMacro(const std::string& name);

        MacroMap& dynamicMacroMap();
        const MacroMap& dynamicMacroMap() const;

        void print(std::ostream& stream);

        types::ObjectPtr resolve();

    public:
        static FunctionMap STATIC_FUNCTION_MAP;
        static MacroMap STATIC_MACRO_MAP;
        static VariableMap STATIC_VARIABLE_MAP;

    private:
        boost::filesystem::path m_sourcePath;
        int m_lineOffset;
        bool m_processed;
        utils::CIterator m_iterator;
        std::ifstream m_input;
        bool m_firstRead;
        FunctionMap m_dynamicFunctionMap;
        MacroMap m_dynamicMacroMap;
        VariableMap m_dynamicVariableMap;
    };
}
#endif   // ----- #ifndef ELEMENTSTACK_INC  -----
