// =======================================================================
// 
//       Filename:  parser.hpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  25.03.2010 21:18:50
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================


#ifndef  PARSER_INC
#define  PARSER_INC
#include "types/string.hpp"
#include "utils/citerator.hpp"
#include "elementstack.hpp"
#include <stdexcept>
#include <map>
#include <list>
#include <boost/shared_ptr.hpp>
#include <boost/filesystem/path.hpp>

namespace wesl {
    // Forward declaration
    namespace types {
        class Object;
    }

    namespace elements {
        class Script;
        class VariableRecall;
        class Element;
        class Command;
    }

    /**
     * This class parses a template file
     * and interprets commands and scripts on the fly.
     * @brief This class parses a template file.
     */
    class Parser {
        public:
            /**
             * @brief Constructs default Parser.
             */
            Parser();

            /**
             * @brief Launches the parsing process.
             * @throw std::logic_error Thrown if a script is detected
             *                         and no rule defined
             * @throw SyntaxError Thrown if an syntax error is detected.
             */
            void parse(ElementStack& stack, int begin = -1);

            int line() const;

        private:
            void safelyIncr();
            void skipWhiteSpace();
            void parseElementStack(ElementStack& elementStack, 
                                   int begin = -1, 
                                   bool ignoreText = false);
            boost::shared_ptr<elements::Element>
                parseDynamicContent(ElementStack& elementStack);
            boost::shared_ptr<elements::Script> 
                parseScript(ElementStack& elementStack);
            boost::shared_ptr<elements::VariableRecall> 
                parseVar(ElementStack& elementStack);
            boost::shared_ptr<elements::Command> 
                parseCommand(ElementStack& elementStack, 
                                    const std::string& cmdName);
            boost::shared_ptr<types::String>
                parseString();
            boost::shared_ptr<types::Object>
                parseNumber();

            utils::CIterator mIterator;
            int m_line;
            boost::filesystem::path m_filePath;
    };
}
#endif   // ----- #ifndef PARSER_INC  -----
