// =======================================================================
// 
//       Filename:  processor.cpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  09.04.2010 22:42:26
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================

#include "processor.hpp"
#include "elements/variablerecall.hpp"
#include "elements/command.hpp"
#include "elements/script.hpp"
#include "symbols/function.hpp"
#include "elements/objectwrapper.hpp"
#include "types/void.hpp"
#include "types/integer.hpp"
#include "types/float.hpp"
#include "process/process.hpp"
#include "utils/citerator.hpp"
#include "exceptions/processingerror.hpp"
#include "callinfo.hpp"
#include <sstream>
#include <fstream>
#include <cstdlib>
#include <boost/regex.hpp>
#include <boost/foreach.hpp>
#include <boost/any.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <algorithm>

#define foreach BOOST_FOREACH

using namespace std;
using namespace boost;
using namespace wesl::elements;
using namespace wesl::symbols;
namespace fs = boost::filesystem;
using wesl::process::Process;
using wesl::exceptions::ProcessingError;

namespace wesl {
    using namespace types;

    Processor::Processor()
    {
        ElementStack::STATIC_FUNCTION_MAP["set"] = 
                            FunctionPtr(new SetFunction());
        ElementStack::STATIC_FUNCTION_MAP["set_script_rule"] = 
                            FunctionPtr(new SetScriptRuleFunction());
    }

    Processor::~Processor() {
    }

    void Processor::processElementStack(ElementStack& stack,
                                        ostream& out)
    {
        stack.print(out);
    }

    types::ObjectPtr Processor::SetFunction::call(CallInfo& callInfo)
    {
        CallInfo::ObjectPointerList& args = callInfo.arguments();
        
        if(args.size() != 2) {
            throw ProcessingError("set: "
                            "invalid number of arguments",
                            callInfo.element().parent().sourcePath(),
                            callInfo.element().line());
        }
        else {
            regex cmdregex("[a-zA-Z_0-9]+");

            if(args[0]->objectType() != types::Object::STRING)
            {
                throw ProcessingError("set: "
                            "invalid type of first argument",
                            callInfo.element().parent().sourcePath(),
                            callInfo.element().line());
            }

            string var = args[0]->toString();
        
            if(!regex_match(var.c_str(), cmdregex)) {
                throw ProcessingError("set: "
                            "invalid variable name",
                            callInfo.element().parent().sourcePath(),
                            callInfo.element().line());
            }
            else {
                callInfo.element().parent().dynamicVariableMap()[var] = 
                                                            args[1];
            }
        }

        return types::VoidPtr(new types::Void());
    }

    types::ObjectPtr Processor::SetScriptRuleFunction::call(
            CallInfo& callInfo)
    {
        CallInfo::ObjectPointerList& args = callInfo.arguments();

        if(args.size() != 2) {
            throw ProcessingError("set_script_rule: "
                    "invalid number of arguments",
                    callInfo.element().parent().sourcePath(),
                    callInfo.element().line());
        }

        string ruleName = args[0]->toString();

        elements::Script::RULES[ruleName] = args[1]->toString();

        return types::VoidPtr(new Void());
    }
}
