// =======================================================================
// 
//       Filename:  processor.hpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  09.04.2010 22:35:13
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================


#ifndef  WESL_PROCESSOR_INC
#define  WESL_PROCESSOR_INC
#include "elementstack.hpp"
#include "symbols/function.hpp"
#include <boost/shared_ptr.hpp>
#include <boost/any.hpp>
#include <stdexcept>
#include <map>
#include <vector>

namespace wesl {

    // Forward declarations
    class CallInfo;

    namespace elements {
        class Command;
        class Script;
        class Text;
        class Variable;
    }

    namespace types {
        class Object;
    }

    namespace navigation {
        class Node;
    }

    namespace utils {
        class CIterator;
    }

    /**
     * @brief This class processes a Page object and
     *        a templatefile::File object and produces
     *        an output file.
     */
    class Processor {
        public:
            /**
             * @brief Constructs a Processor with scriptConfig.
             * @param scriptConfig The script interpreter rules,
             *          used by this instance.
             */
            Processor();
            ~Processor();

            /**
             * @brief Process a templatefile::File object and
             *        a Page object to produce an output
             *        file.
             * @param[in] file The templatefile::File object
             *                 produced by the Parser
             * @param[in] page The information to fill
             *                 the templatefile
             */
            void processElementStack(ElementStack& stack,
                                     std::ostream& out);

        protected:
            class SetFunction : public symbols::Function
            {
            public:
                types::ObjectPtr call(CallInfo& callInfo);
            };

            class SetScriptRuleFunction : public symbols::Function
            {
            public:
                types::ObjectPtr call(CallInfo& callInfo);
            };
    };
}
#endif   // ----- #ifndef WESL_PROCESSOR_INC  -----
