// =======================================================================
// 
//       Filename:  navigation.cpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  03.06.2010 14:10:42
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================

#include "navigation.hpp"
#include "../page/page.hpp"
#include "../navigation/node.hpp"
#include <wesl/callinfo.hpp>
#include <wesl/types/string.hpp>
#include <wesl/elements/command.hpp>
#include <wesl/elements/objectwrapper.hpp>
#include <wesl/exceptions/processingerror.hpp>
#include <boost/regex.hpp>
#include <boost/foreach.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>

#define foreach BOOST_FOREACH

namespace fs = boost::filesystem;
using namespace std;
using namespace boost;

namespace webplodder {
    namespace functions {
        wesl::types::ObjectPtr 
            Navigation::call(wesl::CallInfo& callInfo)
        {
            wesl::CallInfo::ObjectPointerList& args = 
                callInfo.arguments();
            shared_ptr<wesl::types::String> buffer(
                                        new wesl::types::String());

            if(args.size() > 1 || args.size() < 1) {
                throw wesl::exceptions::ProcessingError("navigation: "
                        "invalid number of arguments",
                        callInfo.element().parent().sourcePath(),
                        callInfo.element().line());
            }
            else {

                if(args[0]->objectType() != wesl::types::Object::STRING)
                {
                    throw wesl::exceptions::ProcessingError("navigation: "
                                "invalid argument type",
                                callInfo.element().parent().sourcePath(),
                                callInfo.element().line());
                }
                else {
                    string pattern = 
                            args[0]->toString();
                    map<string,string>::iterator it;
                    regex titleRegex("\\{title\\}");
                    regex hrefRegex("\\{href\\}");
                    regex activeRegex("\\{active\\}");

                    foreach(navigation::Node& node, page().navigation())
                    {
                        string impl;

                        impl = regex_replace(pattern,
                                titleRegex,
                                node.title());
                        impl = regex_replace(impl,
                                hrefRegex,
                                node.href().string());

                        if(fs::equivalent(node.href(), 
                                          page().outputPath()))
                            impl = regex_replace(impl,
                                    activeRegex,
                                    "id='active'");
                        else
                            impl = regex_replace(impl,
                                    activeRegex,
                                    "");
                        buffer->append(impl);
                    }

                }
            }

            return buffer;
        }
    }
}
