// =======================================================================
// 
//       Filename:  node.hpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  25.04.2010 14:56:22
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================


#ifndef  NODE_INC
#define  NODE_INC
#include <iostream>
#include <boost/filesystem/path.hpp>

namespace webplodder {
    namespace navigation {
        /**
         * @brief Represents a Page in the navigation.
         */
        class Node {
        public:
            /**
             * @brief Constructs Node instance.
             * @param title The page title
             * @param href The link to page
             * @param pos The position in the navigation
             */
            Node(const std::string& title,
                    const boost::filesystem::path& href,
                    int pos = 0);

            /**
             * @brief Returns the title of the page.
             */
            std::string title() const;

            /**
             * @brief Returns the link to the page.
             */
            boost::filesystem::path href() const;

            /**
             * @brief Returns the position in the navigation.
             */
            int pos() const;

        private:
            std::string m_title;
            boost::filesystem::path m_href;
            int         m_pos;
        };
    }
}
#endif   // ----- #ifndef NODE_INC  -----
