// =======================================================================
// 
//       Filename:  parser.cpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  10.05.2010 16:49:12
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================

#include "parser.hpp"
#include "page.hpp"
#include <wesl/parser.hpp>
#include <wesl/exceptions/fileerror.hpp>
#include <wesl/types/string.hpp>
#include <fstream>
#include <yaml.h>
#include <iostream>
#include <boost/regex.hpp>
#include <boost/shared_ptr.hpp>

using namespace std;
using namespace boost;
using wesl::exceptions::FileError;

namespace webplodder {
    namespace page {
        Parser::Parser() {

        }

        void Parser::parse(Page& page) {
            ifstream in(page.sourcePath().string().c_str());

            if(!in.good())
                throw FileError("failed to open",
                                page.sourcePath());

            YAML::Parser parser(in);

            YAML::Node doc;

            parser.GetNextDocument(doc);
            YAML::Iterator it;

            string key;

            const YAML::Node *pName = 0;
            if((pName = doc.FindValue("pos")) != 0)
                *pName >> page.m_pos;
            if((pName = doc.FindValue("in_menu")) != 0)
                *pName >> page.m_inMenu;
            if((pName = doc.FindValue("title")) != 0)
                *pName >> page.m_title;

            in.close();
        }

        void Parser::parseBody(Page& page, 
                wesl::ElementStack::VariableMap& variableMap)
        {
            ifstream in(page.sourcePath().string().c_str());

            if(!in.good())
                throw FileError("failed to open",
                        page.sourcePath());

            YAML::Parser parser(in);

            YAML::Node doc;

            parser.GetNextDocument(doc);
            YAML::Iterator it;

            string key;

            for(it = doc.begin(); it != doc.end(); ++it) {
                shared_ptr<wesl::types::String> value(
                                        new wesl::types::String());

                it.first() >> key;
                it.second() >> *value;
                
                variableMap[key] = value;
            }

            in.seekg(0, ifstream::beg);
            wesl::utils::CIterator iter(in);

            boost::regex endRegex("^\\-\\-\\-[ \t]*$");

            bool hasContent = false;
            int offset = 0;

            while(iter.good()) {
                char ch;
                string line;

                while(iter.good()) {
                    ch = iter.currentChar();
                    iter.moveForward();

                    if(ch == '\n')
                        break;

                    line.push_back(ch);
                }

                if(boost::regex_match(line, endRegex)) {
                    hasContent = true;

                    ++offset;
                    break;
                }
                ++offset;
            }

            if(hasContent) {
                wesl::Parser wpParser;

                wpParser.parse(*page.content(), iter.position());
            }

            in.close();
        }
    }
}
