// =======================================================================
// 
//       Filename:  plodder.hpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  27.03.2010 18:00:28
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================


#ifndef  PLODDER_INC
#define  PLODDER_INC
#include <iostream>
#include <stdexcept>
#include <map>
#include <yaml.h>
#include <boost/filesystem/path.hpp>

namespace webplodder {
    // Forward declarations
    class Processor;

    namespace utils {
        class PathList;
    }

    class Plodder {

        friend class Processor;

    public:
        /**
         * The config directory includes the config file
         * including the specifications for the script parsing.
         * @brief Constructs a Plodder up on a config directory.
         * @param[in] configDir This is the path of the config directory
         *            which includes a subdirectory `plugins'
         *            with all available Plugins.
         *            It also includes a config file with the
         *            script specifications
         */
        Plodder(const boost::filesystem::path& configDir);

        ~Plodder();

        /**
         * It parses the configfile in the configDir
         * @brief Initializes the Plodder.
         * @throw std::runtime_error Thrown if an error occurs
         *                           while opening or reading the
         *                           configfile
         * @throw YAML::Exception Thrown if an error occurs
         *          while parsing the configfile
         */
        void init();

        /**
         * Searches it for suitable
         * *.page and *.template files and processes
         * them.
         * @brief Scans a target directory.
         * @throw std::runtime_error Thrown if no *.page
         *                           or *.template file found
         */
        void scanDir(const boost::filesystem::path& dir,
                     utils::PathList& exclude);

    protected:
        std::string formatPath(const boost::filesystem::path& filePath);

    private:
        boost::filesystem::path m_targetPath;
        boost::filesystem::path m_configDir;
        boost::shared_ptr<Processor> m_processor;
    };
}
#endif   // ----- #ifndef PLODDER_INC  -----
