// =======================================================================
// 
//       Filename:  processor.cpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  09.04.2010 22:42:26
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================

#include "processor.hpp"
#include "page/page.hpp"
#include "pluginmanager.hpp"
#include "plugin.hpp"
#include "navigation/node.hpp"
#include "exceptions/systemerror.hpp"
#include "exceptions/pluginerror.hpp"
#include "functions/pagecontent.hpp"
#include "functions/navigation.hpp"
#include "functions/title.hpp"
#include <wesl/elementstack.hpp>
#include <wesl/exceptions/fileerror.hpp>
#include <wesl/callinfo.hpp>
#include <wesl/elements/command.hpp>
#include <wesl/exceptions/processingerror.hpp>
#include <fstream>
#include <cstdlib>
#include <boost/regex.hpp>
#include <boost/foreach.hpp>
#include <boost/any.hpp>
#include <boost/filesystem/path.hpp>
#include <algorithm>

#define foreach BOOST_FOREACH

using namespace std;
using namespace boost;
namespace fs = boost::filesystem;
using webplodder::exceptions::SystemError;
using wesl::exceptions::FileError;
using webplodder::exceptions::PluginError;

namespace webplodder {

    Processor::Processor()
        : m_page(0)
    {
        m_wpFunctionMap["navigation"] = shared_ptr<functions::Navigation>(
                                            new functions::Navigation());
        m_wpMacroMap["page_content"] = shared_ptr<
                                                functions::PageContent>(
                                            new functions::PageContent());
        m_wpFunctionMap["title"] = shared_ptr<functions::Title>(
                                            new functions::Title());
        m_wpFunctionMap["*"] = shared_ptr<DefaultFunction>(
                                            new DefaultFunction());

        WpFunctionMap::iterator fIter = m_wpFunctionMap.begin();
        for(; fIter != m_wpFunctionMap.end(); ++fIter) {
            wesl::ElementStack::STATIC_FUNCTION_MAP[fIter->first] = 
                fIter->second;
        }

        WpMacroMap::iterator mIter = m_wpMacroMap.begin();
        for(; mIter != m_wpMacroMap.end(); ++mIter) {
            wesl::ElementStack::STATIC_MACRO_MAP[mIter->first] = 
                mIter->second;
        }
    }

    Processor::~Processor() {
    }

    void Processor::process(page::Page& page,
                            wesl::ElementStack& elementStack)
    {
        WpFunctionMap::iterator fIter = m_wpFunctionMap.begin();
        for(; fIter != m_wpFunctionMap.end(); ++fIter) {
            dynamic_pointer_cast<DefaultFunction>(
                                         fIter->second)->setPage(page);
        }

        WpMacroMap::iterator mIter = m_wpMacroMap.begin();
        for(; mIter != m_wpMacroMap.end(); ++mIter) {
            dynamic_pointer_cast<DefaultMacro>(
                                          mIter->second)->setPage(page);
        }

        m_page = &page;

        const char* outputFileName = 
                    page.outputPath().string().c_str();
        ofstream out(outputFileName);

        if(!out.good())
            throw FileError("unable to open", 
                              page.outputPath());

        processElementStack(elementStack, out);

        out.close();
    }

    void Processor::installPluginManager(shared_ptr<PluginManager> 
                                                                manager)
    {
        m_pluginManager = manager;
    }

    wesl::types::ObjectPtr 
        Processor::DefaultFunction::call(
            wesl::CallInfo& callInfo)
    {
        wesl::types::ObjectPtr objectPtr;

        try {
            shared_ptr<Plugin> plugin =
                m_pluginManager->getPlugin(callInfo.element().name());

            plugin->call(page(), callInfo);
        }
        catch(PluginError& e) {
            throw wesl::exceptions::ProcessingError("not a function: " + 
                        callInfo.element().name(), 
                        callInfo.element().parent().sourcePath(),
                        callInfo.element().line());
        }

        return objectPtr;
    }

    void Processor::DefaultFunction::setPage(page::Page& page) {
        m_page = &page;
    }

    page::Page& Processor::DefaultFunction::page() {
        if(m_page == 0)
            throw logic_error("webplodder::Processor::"
                              "DefaultFunction::page: No Page set");
        else
            return *m_page;
    }

    void Processor::DefaultMacro::setPage(page::Page& page) {
        m_page = &page;
    }

    page::Page& Processor::DefaultMacro::page() {
        if(m_page == 0)
            throw logic_error("webplodder::Processor::"
                              "DefaultMacro::page: No Page set");
        else
            return *m_page;
    }

    shared_ptr<PluginManager> Processor::m_pluginManager = 
                                    shared_ptr<PluginManager>();
}
