// =======================================================================
// 
//       Filename:  processor.hpp
// 
//    Description:  
// 
//        Version:  1.0
//        Created:  09.04.2010 22:35:13
//       Revision:  none
//       Compiler:  g++
// 
//         Author:  Dominik 'domachine' Burgdörfer (-), dominik.burgdoerfer@googlemail.com
//        Company:  -
// 
// =======================================================================


#ifndef  PROCESSOR_INC
#define  PROCESSOR_INC
#include <wesl/processor.hpp>
#include <wesl/symbols/function.hpp>
#include <wesl/symbols/macro.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/any.hpp>
#include <stdexcept>
#include <map>
#include <vector>

namespace webplodder {

    // Forward declarations
    class Plodder;

    namespace page {
        class Page;
    }

    class Plugin;
    class PluginManager;

    namespace elements {
        class Command;
        class Script;
        class Text;
    }

    namespace navigation {
        class Node;
    }

    namespace utils {
        class CIterator;
    }

    /**
     * @brief This class processes a Page object and
     *        a templatefile::File object and produces
     *        an output file.
     */
    class Processor : public wesl::Processor {

        public:
            /**
             * @brief Constructs a Processor with scriptConfig.
             * @param scriptConfig The script interpreter rules,
             *          used by this instance.
             */
            Processor();
            ~Processor();

            /**
             * @brief Process a templatefile::File object and
             *        a Page object to produce an output
             *        file.
             * @param[in] file The templatefile::File object
             *                 produced by the Parser
             * @param[in] page The information to fill
             *                 the templatefile
             */
            void process(page::Page& page,
                         wesl::ElementStack& elementStack);

            /**
             * @brief Installs a PluginManager.
             */
            void installPluginManager(boost::shared_ptr<PluginManager> 
                                                    manager);

            // Builtin Functions
            class DefaultFunction : public wesl::symbols::Function
            {
            public:
                wesl::types::ObjectPtr 
                        call(wesl::CallInfo& callInfo);

                void setPage(page::Page& page);
                page::Page& page();

            private:
                page::Page* m_page;
            };

            class DefaultMacro : public wesl::symbols::Macro
            {
            public:
                virtual void call(wesl::CallInfo& callInfo, std::ostream& stream) = 0;

                void setPage(page::Page& page);
                page::Page& page();

            private:
                page::Page* m_page;
            };

        private:
            typedef std::map<std::string,boost::shared_ptr<
                                    DefaultFunction> > WpFunctionMap;
            typedef std::map<std::string,boost::shared_ptr<
                                    DefaultMacro> > WpMacroMap;


            WpFunctionMap m_wpFunctionMap;
            WpMacroMap m_wpMacroMap;
            static boost::shared_ptr<PluginManager> m_pluginManager;
            page::Page* m_page;
    };
}
#endif   // ----- #ifndef PROCESSOR_INC  -----
